/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package activity;

import java.util.HashMap;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.InterruptedPullException;
import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.control.activity.Util;
import org.objectweb.dream.control.activity.manager.ThreadPoolManager;
import org.objectweb.dream.control.activity.task.AbstractTask;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.message.Message;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Client component implementation
 */
public class ClientImpl extends AbstractComponent
{

  protected Pull           inPullItf;
  protected MessageManager messageManagerItf;

  // ---------------------------------------------------------------------------
  // Overridden methods
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractComponent#beforeFirstStart(Component)
   */
  protected void beforeFirstStart(Component componentItf)
      throws IllegalLifeCycleException
  {
    try
    {
      HashMap<String, Object> context = new HashMap<String, Object>();
      context.put("thread", "pool");
      Util.addTask(componentItf, new PullTask(), context);
      logger.log(BasicLevel.INFO, "task added");
    }
    catch (Exception e)
    {
      throw new IllegalLifeCycleException("Can't add task");
    }
  }

  protected class PullTask extends AbstractTask
  {

    PullTask()
    {
      super("PullTask");
    }

    // -------------------------------------------------------------------------
    // Implementation of the Task interface
    // -------------------------------------------------------------------------

    /**
     * @see org.objectweb.dream.control.activity.task.Task#execute(Object)
     */
    public Object execute(Object hints) throws InterruptedException
    {
      Message message = null;
      try
      {
        message = inPullItf.pull();
        System.out.println("PullComponent has received a message : " + message);
        messageManagerItf.deleteMessage(message);
      }
      catch (PullException e)
      {
        System.out.println("An exception occurred while pulling a message : "
            + e);
        if (e instanceof InterruptedPullException)
        {
          System.out.println("The pull method has been interrupted");
        }
        return STOP_EXECUTING;
      }
      return EXECUTE_AGAIN;
    }

    // -------------------------------------------------------------------------
    // Overridden method
    // -------------------------------------------------------------------------

    /**
     * @see org.objectweb.dream.control.activity.task.Task#registered(Object)
     */
    public void registered(Object controlItf)
    {
      if (controlItf instanceof ThreadPoolManager)
      {
        ThreadPoolManager tpm = (ThreadPoolManager) controlItf;
        try
        {
          tpm.addThread(this);
        }
        catch (Exception e)
        {
          e.printStackTrace();
        }
      }
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Pull.IN_PULL_ITF_NAME))
    {
      inPullItf = (Pull) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Pull.IN_PULL_ITF_NAME, MessageManager.ITF_NAME};
  }

}
