/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream;

import org.objectweb.dream.message.Message;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * This component forwards calls received on its <code>Push</code> input to
 * its <code>Push1</code> output.
 */
public class PushToPush1Impl extends AbstractComponent implements Push
{

  /** The Push output. */
  Push1 outPush1Itf;

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Push1#push1(Message)
   */
  public void push(Message message) throws PushException
  {
    outPush1Itf.push1(message);
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Push1.OUT_PUSH_ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
   *      java.lang.Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Push1.OUT_PUSH_ITF_NAME))
    {
      outPush1Itf = (Push1) serverItf;
    }
  }
}