/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Matthieu Leclercq
 */

package org.objectweb.dream.control.activity.manager;

import java.util.Map;

import org.objectweb.dream.control.activity.task.IllegalTaskException;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.control.activity.task.TaskStoppedListener;

/**
 * A control interface to manage tasks.
 */
public interface TaskManager
{
  /** Default name of this interface. */
  String ITF_NAME = "task-manager";

  /**
   * Registers a new task.
   * 
   * @param task the task to be registered.
   * @param hints additionnal parameters.
   * @throws IllegalTaskException if an error occurs.
   */
  void registerTask(Task task, Map hints) throws IllegalTaskException;

  /**
   * Unregisters the specified task.
   * 
   * @param task the task to be unregistered.
   * @throws IllegalTaskException if an error occurs.
   * @throws InterruptedException if the method is interrupted
   */
  void unregisterTask(Task task) throws IllegalTaskException,
      InterruptedException;

  /**
   * Interrupts the specified task. An interrupted task is no more executed. The
   * given <code>listener</code> is notified when the task is stopped. The
   * interrupted task should then be unregistered from the activity manager.
   * 
   * @param task the task to interrupt.
   * @param listener the listener that is notified when the task is interrupted.
   *          Can be <code>null</code>.
   * @throws IllegalTaskException if the given task is unknown.
   */
  void interruptTask(Task task, TaskStoppedListener listener)
      throws IllegalTaskException;

  /**
   * Returns the tasks that have been registered.
   * 
   * @return the tasks that have been registered.
   */
  Task[] getTasks();
}