/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

import org.objectweb.dream.pool.ObjectPool;
import org.objectweb.dream.pool.ObjectPoolSoftReferenceImpl;
import org.objectweb.dream.pool.Recyclable;
import org.objectweb.dream.pool.RecyclableFactory;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Extension of the basic message manager. This implementation recycle messages
 * and chunks in object pools.
 */
public class MessageManagerPoolingImpl extends MessageManagerImpl
    implements
      RecyclableFactory
{
  ObjectPool messagePoolItf;

  // ---------------------------------------------------------------------------
  // Overridden MessageManager methods
  // ---------------------------------------------------------------------------

  /**
   * @see MessageManager#createMessage()
   */
  @Override
  public Message createMessage()
  {
    Message message = (Message) messagePoolItf.newInstance();
    message.incrementReferenceCounter();
    return message;
  }

  /**
   * @see MessageManagerImpl#recycleMessageInstance(Message)
   */
  @Override
  protected void recycleMessageInstance(final Message message)
  {
    messagePoolItf.recycleInstance(message);
  }

  /**
   * @see MessageManagerImpl#createChunkFactory(Class)
   */
  @Override
  protected <T extends AbstractChunk<T>> ChunkFactoryReference<T> createChunkFactory(
      final Class<T> chunkType)
  {
    ObjectPoolSoftReferenceImpl pool = new ObjectPoolSoftReferenceImpl();
    ChunkFactoryReference<T> factory;
    try
    {
      factory = new BasicChunkFactoryPoolingImpl<T>(chunkType, this, pool);
    }
    catch (Exception e)
    {
      throw new IllegalArgumentException(e);
    }
    try
    {
      pool.bindFc(RecyclableFactory.ITF_NAME, factory);
    }
    catch (Exception e)
    {
      Error.bug(logger, e);
    }
    return factory;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the RecyclableFactory interface
  // ---------------------------------------------------------------------------

  /**
   * @see RecyclableFactory#newInstance()
   */
  public Recyclable newInstance()
  {
    return new BasicMessageImpl(this);
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{ObjectPool.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  @Override
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(ObjectPool.ITF_NAME))
    {
      messagePoolItf = (ObjectPool) serverItf;
    }
  }

}
