/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import org.objectweb.fractal.api.control.AttributeController;

/**
 * Attribute controller interface for
 * {@link MessageCodecHelperObjectStreamImpl }message codec helper component.
 * Defines a <code>reuseObjectStream</code> boolean attribute that control the
 * "life cycle" of the object streams used by the codec. If <code>false</code>,
 * the codec will open a new Object stream for each encoding or decoding
 * operation. Else, the codec resuse the already opened Object stream (see
 * {@link CodecInputOutput#setInput(Object) }and
 * {@link CodecInputOutput#setOutput(Object)}).<br>
 * Warning : if <code>true</code>, you must be sure that the
 * {@link java.io.ObjectOutputStream#writeObject(Object) }(respectivly
 * <code>readObject</code>) method is never used, unless a memory leak can
 * happen. Inded, the <code>writeObject</code> method keeps a reference to
 * each serialized object. You must use the
 * {@link org.objectweb.dream.util.Util#writeObject(java.io.ObjectOutput, Object)}
 * (respectivly,
 * {@link org.objectweb.dream.util.Util#readObject(java.io.ObjectInput)})
 * instead. This method will use the
 * {@link java.io.ObjectOutputStream#writeUnshared(Object) }method
 * (respectivly, {@link java.io.ObjectInputStream#readUnshared()}).<br>
 * To do so, you must implements specialized serialization method in your chunk
 * and message implementations, and in any serialized object.
 * 
 * @see java.io.Serializable
 * @see java.io.Externalizable
 * @see java.io.ObjectOutputStream
 * @see java.io.ObjectInputStream
 */
public interface MessageCodecHelperObjectStreamAttributeController
    extends
      AttributeController
{
  /**
   * Returns the <code>reuseObjectStream</code> attribute value.
   * 
   * @return the <code>reuseObjectStream</code> attribute value.
   */
  boolean getReuseObjectStream();

  /**
   * Sets the <code>reuseObjectStream</code> attribute.
   * 
   * @param reuseObjectStream the new <code>reuseObjectStream</code> attribute
   *          value.
   */
  void setReuseObjectStream(boolean reuseObjectStream);
}