/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

public class ChunkTest extends AbstractChunk<ChunkTest>
{
  private static final long   serialVersionUID = 6804459537753877342L;
  private static int          nextNum          = 0;
  private static final Object LOCK             = new Object();

  private int                 num;

  public ChunkTest()
  {
    synchronized (LOCK)
    {
      num = nextNum++;
    }
  }

  public ChunkTest(int num)
  {
    this.num = num;
  }

  public int getNum()
  {
    return num;
  }

  public void setNum(int num)
  {
    this.num = num;
  }

  /**
   * @see Object#equals(Object)
   */
  public boolean equals(Object obj)
  {
    if (!(obj instanceof ChunkTest))
    {
      return false;
    }
    return ((ChunkTest) obj).num == num;
  }

  /**
   * @see java.lang.Object#hashCode()
   */
  public int hashCode()
  {
    return num;
  }

  @Override
  protected ChunkTest newChunk()
  {
    return new ChunkTest();
  }

  @Override
  protected void transfertStateTo(ChunkTest newInstance)
  {
    newInstance.num = num;
  }

  /**
   * @see org.objectweb.dream.pool.Recyclable#recycle()
   */
  public void recycle()
  {
    num = 0;
  }

  public void writeExternal(ObjectOutput out) throws IOException
  {
    out.writeInt(num);
  }

  public void readExternal(ObjectInput in) throws IOException,
      ClassNotFoundException
  {
    num = in.readInt();
  }
}
