/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

import java.util.HashMap;

import org.objectweb.dream.AbstractPerf;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestMessageManager extends AbstractPerf
{
  private static enum TestVersion {
    TEST2, TEST3
  };

  private static final int                           WARMUP_LOOP    = 2000000;

  private static final int                           MAX_BURST_SIZE = 500;

  private org.objectweb.dream.message.MessageManager messageManagerItf2;

  private Message[]                         burstTab2;

  private ChunkFactoryReference<PerfTestChunk>       chunkFactory;

  private TestVersion                                testVersion;

  protected void setUp() throws Exception
  {
    System.out.println();
    super.setUp();
    burstTab2 = new Message[MAX_BURST_SIZE];
  }

  public void testMessageManager2() throws Exception
  {
    testVersion = TestVersion.TEST2;
    init2("New Version. Without instance pooling",
        "org.objectweb.dream.message.MessageManager");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  public void testMessageManager3() throws Exception
  {
    testVersion = TestVersion.TEST2;
    init2("New Version. With instance pooling and chunk recycling",
        "org.objectweb.dream.message.MessageManagerPooling");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  public void testMessageManager4() throws Exception
  {
    testVersion = TestVersion.TEST3;
    init2("New Version. With instance pooling and without chunk recycling",
        "org.objectweb.dream.message.MessageManagerPooling");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  private void init2(String context, String adl) throws Exception
  {
    if (context != null)
    {
      System.out.println(context);
    }
    testComponent = (Component) FactoryFactory.getFactory().newComponent(adl,
        new HashMap());
    Fractal.getLifeCycleController(testComponent).startFc();
    messageManagerItf2 = (org.objectweb.dream.message.MessageManager) testComponent
        .getFcInterface(MessageManager.ITF_NAME);
    chunkFactory = messageManagerItf2.getChunkFactory(PerfTestChunk.class);

    warmup();
  }

  private void warmup() throws Exception
  {
    System.err.print("warming up...");
    System.err.flush();
    runPerf1(1, WARMUP_LOOP);
    System.err.println(" OK");
  }

  protected void runPerf1(int burstSize, int nbBurst)
  {
    switch (testVersion)
    {
      case TEST2 :
        runPerfV2(burstSize, nbBurst);
        break;
      case TEST3 :
        runPerfV3(burstSize, nbBurst);
        break;
    }
  }

  protected void runPerfV2(int burstSize, int nbBurst)
  {
    for (int burst = 0; burst < nbBurst; burst++)
    {
      for (int i = 0; i < burstSize; i++)
      {
        burstTab2[i] = messageManagerItf2.createMessage();
      }
      for (int i = 0; i < burstSize; i++)
      {
        messageManagerItf2.addChunk(burstTab2[i], "toto", messageManagerItf2
            .createChunk(chunkFactory));
      }
      for (int i = 0; i < burstSize; i++)
      {
        PerfTestChunk chunk = (PerfTestChunk) messageManagerItf2.removeChunk(
            burstTab2[i], "toto");
        messageManagerItf2.deleteChunk(chunk);
      }
      for (int i = 0; i < burstSize; i++)
      {
        messageManagerItf2.deleteMessage(burstTab2[i]);
        burstTab2[i] = null;
      }
    }
  }

  protected void runPerfV3(int burstSize, int nbBurst)
  {
    for (int burst = 0; burst < nbBurst; burst++)
    {
      for (int i = 0; i < burstSize; i++)
      {
        burstTab2[i] = messageManagerItf2.createMessage();
      }
      for (int i = 0; i < burstSize; i++)
      {
        messageManagerItf2.addChunk(burstTab2[i], "toto", messageManagerItf2
            .createChunk(chunkFactory));
      }
      for (int i = 0; i < burstSize; i++)
      {
        @SuppressWarnings("unused")
        PerfTestChunk chunk = (PerfTestChunk) messageManagerItf2.removeChunk(
            burstTab2[i], "toto");
      }
      for (int i = 0; i < burstSize; i++)
      {
        messageManagerItf2.deleteMessage(burstTab2[i]);
        burstTab2[i] = null;
      }
    }
  }
}
