/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.pool;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.AbstractPerf;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestObjectPool extends AbstractPerf
{
  private static final int    WARMUP_LOOP             = 20000000;
  private static final String OBJECT_POOL             = "org.objectweb.dream.pool.TestObjectPool";
  private static final String INTERCEPTED_OBJECT_POOL = "org.objectweb.dream.pool.TestObjectPoolIntercepted";

  private static final int    MAX_BURST_SIZE          = 500;

  private ObjectPool          objectPoolItf;

  private RecyclableBench[]   burstTab;

  protected void setUp() throws Exception
  {
    System.out.println();
    super.setUp();
    burstTab = new RecyclableBench[MAX_BURST_SIZE];
  }

  public void testObjectPool1() throws Exception
  {
    init("Basic instance pooling. Without interceptor", OBJECT_POOL,
        "org.objectweb.dream.pool.ObjectPool(500)");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  public void testObjectPool2() throws Exception
  {
    init("Basic instance pooling. With interceptor", INTERCEPTED_OBJECT_POOL,
        "org.objectweb.dream.pool.ObjectPool(500)");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  public void testObjectPoolSoft1() throws Exception
  {
    init("Instance pooling with soft reference. Without interceptor",
        OBJECT_POOL, "org.objectweb.dream.pool.ObjectPoolSoftReference");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  public void testObjectPoolSoft2() throws Exception
  {
    init("Instance pooling with soft reference. With interceptor",
        INTERCEPTED_OBJECT_POOL,
        "org.objectweb.dream.pool.ObjectPoolSoftReference");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  public void testObjectNew1() throws Exception
  {
    init("Instance creation (pool size=0). Without interceptor", OBJECT_POOL,
        "org.objectweb.dream.pool.ObjectPool(0)");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  public void testObjectNew2() throws Exception
  {
    init("Instance creation (pool size=0). With interceptor",
        INTERCEPTED_OBJECT_POOL, "org.objectweb.dream.pool.ObjectPool(0)");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  public void testRecyclableBenchIntNew1() throws Exception
  {
    init("Specific Instance creation (new Object()). Without interceptor",
        OBJECT_POOL, "org.objectweb.dream.pool.RecyclableBenchIntNew");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  public void testRecyclableBenchIntNew2() throws Exception
  {
    init("Specific Instance creation (new Object()). With interceptor",
        INTERCEPTED_OBJECT_POOL,
        "org.objectweb.dream.pool.RecyclableBenchIntNew");
    runPerf(1, 10000000);
    runPerf(5, 2000000);
    runPerf(10, 1000000);
    runPerf(50, 200000);
    runPerf(500, 20000);
  }

  private void init(String context, String adl, String poolADL)
      throws Exception
  {
    if (context != null)
    {
      System.out.println(context);
    }
    System.out.println(poolADL);
    Map<String, String> hints = new HashMap<String, String>();
    hints.put("poolADL", poolADL);
    testComponent = (Component) FactoryFactory.getFactory().newComponent(adl,
        hints);
    Fractal.getLifeCycleController(testComponent).startFc();
    objectPoolItf = (ObjectPool) testComponent
        .getFcInterface(ObjectPool.ITF_NAME);

    warmup();
  }

  private void warmup() throws Exception
  {
    System.err.print("warming up...");
    System.err.flush();
    runPerf1(1, WARMUP_LOOP);
    System.err.println(" OK");
  }

  protected void runPerf1(int burstSize, int nbBurst)
  {
    for (int burst = 0; burst < nbBurst; burst++)
    {
      for (int i = 0; i < burstSize; i++)
      {
        burstTab[i] = (RecyclableBench) objectPoolItf.newInstance();
      }
      for (int i = 0; i < burstSize; i++)
      {
        burstTab[i].doSomething();
      }
      for (int i = 0; i < burstSize; i++)
      {
        objectPoolItf.recycleInstance(burstTab[i]);
        burstTab[i] = null;
      }
    }
  }
}
