/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.adl;

import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Compiler;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.TaskMap;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * A simple compiler with two {@link org.objectweb.fractal.adl.Compiler }client
 * interfaces :<code>precompile</code> and <code>compile</code>. These two
 * interfaces are called sequencialy.
 */
public class PrecompilerCompiler implements BindingController, Compiler
{

  /** The name of the client interface used to precompile */
  public static final String PRECOMPILE_ITF_NAME = "pre-compiler";
  /** The name of the client interface used to compile */
  public static final String COMPILE_ITF_NAME    = "post-compiler";

  protected Compiler         precompileItf;
  protected Compiler         compileItf;

  /**
   * @see org.objectweb.fractal.adl.Compiler#compile(org.objectweb.fractal.adl.Definition,
   *      org.objectweb.fractal.adl.TaskMap, java.util.Map)
   */
  public void compile(Definition definition, TaskMap tasks, Map context)
      throws ADLException
  {
    precompileItf.compile(definition, tasks, context);
    compileItf.compile(definition, tasks, context);
  }

  // --------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // --------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{PRECOMPILE_ITF_NAME, COMPILE_ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
   */
  public Object lookupFc(String clientItfName) throws NoSuchInterfaceException
  {
    if (PRECOMPILE_ITF_NAME.equals(clientItfName))
    {
      return precompileItf;
    }
    else if (COMPILE_ITF_NAME.equals(clientItfName))
    {
      return compileItf;
    }
    else
    {
      return null;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
   *      java.lang.Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    if (PRECOMPILE_ITF_NAME.equals(clientItfName))
    {
      precompileItf = (Compiler) serverItf;
    }
    else if (COMPILE_ITF_NAME.equals(clientItfName))
    {
      compileItf = (Compiler) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
   */
  public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException
  {
    if (PRECOMPILE_ITF_NAME.equals(clientItfName))
    {
      precompileItf = null;
    }
    else if (COMPILE_ITF_NAME.equals(clientItfName))
    {
      compileItf = null;
    }
  }

}